/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2009 Jaroslav Tulach
 */

package cz.xelfi.quoridor.emailer

import java.net.URL
import java.util.ResourceBundle
import java.text.MessageFormat
import scala.collection.Map
import scala.collection.mutable.HashMap

abstract class Notify {
  protected def allPlayers(): Map[String,List[Info]]
  protected def emails(ids : Collection[String]): Map[String,String]
  protected def sendEmail(address : String, subject : String, text : String) : Unit

  case class Info(lastMove : Long, gameId : String) {
  }

  def process() {
    val ret = allPlayers()
    val name2Address = emails(ret.keySet)

    for (val player <- ret.keySet) {
        val address = name2Address.getOrElse(player, null)

        val rb = ResourceBundle.getBundle("cz.xelfi.quoridor.emailer.Bundle")
        val sb = new StringBuilder
        sb.append(rb.getString("MSG_HEADER"))
        var subject = rb.getString("MSG_SUBJECT")
        val mf = new MessageFormat(rb.getString("MSG_GAME"))
        for (val info <- ret(player)) {
          val delay : Long = (System.currentTimeMillis - info.lastMove) / (3600 * 1000 * 24)
          if (delay > 14) {
            subject = rb.getString("MSG_SUBJECT_WARN")
          }
          val arr = new Array[Object](2);
          arr(0) = "http://quoridor.xelfi.cz/games/" + info.gameId
          arr(1) = java.lang.Long.valueOf(delay)
          sb.append(mf.format(arr))
        }
        if (address != null) {
          sendEmail(
              "quoridor@xelfi.cz",
              subject,
              sb.toString
          )
        }
    }
  }
}
