/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2009 Jaroslav Tulach
 */

package cz.xelfi.quoridor.webidor;

import cz.xelfi.quoridor.Board;
import cz.xelfi.quoridor.IllegalPositionException;
import cz.xelfi.quoridor.Move;
import cz.xelfi.quoridor.Player;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.adapters.XmlAdapter;
import javax.xml.bind.annotation.adapters.XmlJavaTypeAdapter;

/**
 *
 * @author Jaroslav Tulach <jtulach@netbeans.org>
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.FIELD)
public final class Game extends Object {
    @XmlElement
    private GameId id;
    @XmlElement
    @XmlJavaTypeAdapter(BoardAdapter.class)
    private Board board;
    @XmlElementWrapper(name="moves")
    @XmlElement(name="item")
    private List<CommentedMove> moves;
    private Integer move;

    Game() {
    }

    public Game(String first, String second) {
        this.id = new GameId(first, second);
        this.board = Board.empty();
    }

    public Game(GameId id) {
        this.id = id;
        this.board = Board.empty();
    }

    public GameId getId() {
        return id;
    }
    
    public Board getBoard() {
        return board;
    }

    @XmlAttribute
    final String getCurrentPlayer() {
        Player w = board.getWinner();
        if(w==null){
            if (board.getCurrentPlayer() == board.getPlayers().get(0)) {
                return id.getWhite();
            } else {
                return id.getBlack();
            }
        }else{
            if (w == board.getPlayers().get(0)) {
                return id.getWhite();
            } else {
                return id.getBlack();
            }
        }
    }

    public void apply(String player, Move m, Date when) throws IllegalPositionException {
        Player p = null;
        if (id.getWhite().equals(player)) {
            p = getBoard().getPlayers().get(0);
        } else {
            if (id.getBlack().equals(player)) {
                p = getBoard().getPlayers().get(1);
            }
        }
        if (p != getBoard().getCurrentPlayer()) {
            throw new IllegalArgumentException("Wrong player: " + player);
        }

        board = getBoard().apply(m);
        if (when == null) {
            when = new Date(id.getModified());
        }
        final GameStatus status = GameStatus.valueOf(board);
        id = new GameId(
            id.getId(), id.getWhite(), id.getBlack(),
            new Date(id.getStarted()), when, status,
            id.getComments(), !status.isInProgress());
        getMoves().add(new CommentedMove(m, getMoves().size() + 1));
    }

    public void comment(String player, String comment, Date date) {
        Note n = new Note(comment, date, player);
        id = new GameId(
            id.getId(), id.getWhite(), id.getBlack(),
            new Date(id.getStarted()), new Date(id.getModified()),
            GameStatus.valueOf(board), id.getComments() + 1, id.isFinished()
        );
        getMoves().get(getMoves().size() - 1).addNote(n);
    }


    public List<CommentedMove> getMoves() {
        if (moves == null) {
            moves = new ArrayList<CommentedMove>();
        }
        return moves;
    }

    @XmlAttribute
    public int getCurrentMove() {
        return move == null ? getMoves().size() : move;
    }

    public Game snapshot(int move) throws IllegalPositionException {
        Board b = Board.empty();
        int cnt = 0;
        for (CommentedMove m : getMoves()) {
            if (move-- <= 0) {
                break;
            }
            b = b.apply(m.getMove());
            cnt++;
        }

        Game g = new Game(
            new GameId(
                id.getId(), id.getWhite(), id.getBlack(),
                new Date(id.getStarted()), new Date(id.getModified()),
                GameStatus.history, id.getComments(), id.isFinished()
            )
        );
        g.board = b;
        g.move = cnt;
        g.moves = new ArrayList<CommentedMove>(getMoves());
        return g;
    }
/*
    private static final class MoveAdapter extends XmlAdapter<CommentedMove[],List<Move>> {
        @Override
        public List<Move> unmarshal(CommentedMove[] arr) throws Exception {
            List<Move> res = new ArrayList<Move>();
            if (arr != null) {
                for (CommentedMove v : arr) {
                    res.add(Move.valueOf(v.move));
                }
            }
            return res;
        }

        @Override
        public CommentedMove[] marshal(List<Move> arr) throws Exception {
            List<CommentedMove> res = new ArrayList<CommentedMove>();
            if (arr != null) {
                int index = 0;
                for (Move m : arr) {
                    res.add(new CommentedMove(m.toString(), ++index));
                }
            }
            return res.toArray(new CommentedMove[0]);
        }
    } // end of MoveAdapter
*/
    private static final class BoardAdapter extends XmlAdapter<String,Board> {

        @Override
        public Board unmarshal(String v) throws Exception {
            return v == null ? null : Board.valueOf(v);
//            return v == null ? null : new Board(v);
        }

        @Override
        public String marshal(Board v) throws Exception {
            return v == null ? null : v.toString();
//            return v == null ? null : Board.board2HashCode(v);
        }

    }
}
